@@ -14,6 +14,7 @@ from pathlib import Path
 from typing import Callable
 
 import pyLARDA.NcReader as NcReader
+import pyLARDA.RPGReader as RPGReader
 import pyLARDA.ParameterInfo as ParameterInfo
 #import pyLARDA.DataBuffer as DataBuffer
 #import pyLARDA.MeteoReader as MeteoReader
@@ -53,7 +54,7 @@ def convert_regex_date_to_dt(re_date):
     if len(re_date['year']) == 2:
         re_date['year'] = '20' + re_date['year']
     for k in ['year', 'month', 'day', 'hour', 'minute', 'second']:
-        if k in re_date.keys():
+        if k in re_date.keys() and re_date[k] is not None:
             l.append(int(re_date[k]))
     return datetime.datetime(*l)
 
@@ -84,13 +85,17 @@ def setupreader(paraminfo) -> Callable:
 
     if paraminfo["ncreader"] == 'timeheight_limrad94':
         reader = NcReader.timeheightreader_rpgfmcw(paraminfo)
+    elif paraminfo["ncreader"] in ['spec_rpg94binary', 'timeheight_rpg94binary', 'time_rpg94binary']:
+        reader = RPGReader.rpgfmcw_binary(paraminfo)
+    elif paraminfo["ncreader"] in ['time_hatprobinary', 'timeheight_hatprobinary']:
+        reader = RPGReader.hatpro_binary(paraminfo)
     elif paraminfo["ncreader"] == 'spec_limrad94':
         reader = NcReader.specreader_rpgfmcw(paraminfo)
     elif paraminfo["ncreader"] == 'spec_rpgpy':
         reader = NcReader.specreader_rpgpy(paraminfo)
     elif paraminfo["ncreader"] == 'spec_kazr':
         reader = NcReader.specreader_kazr(paraminfo)
-    elif paraminfo["ncreader"] in ['aux', 'aux_all_ts']:
+    elif paraminfo["ncreader"] in ['aux', 'aux_all_ts', 'aux_ts_slice']:
         reader = NcReader.auxreader(paraminfo)
     elif paraminfo["ncreader"] in ['scan_timeheight', 'scan_time']:
         reader = NcReader.scanreader_mira(paraminfo) 
@@ -107,6 +112,8 @@ def setupreader(paraminfo) -> Callable:
         reader = NcReader.reader(paraminfo)
     elif paraminfo["ncreader"] == "wyoming_sounding_txt":
         reader = NcReader.reader_wyoming_sounding(paraminfo)
+    elif paraminfo["ncreader"] == 'psd':
+        reader = NcReader.psd_reader(paraminfo)
     else:
         reader = NcReader.reader(paraminfo)
 
@@ -132,12 +139,27 @@ def setup_valid_date_filter(valid_dates) -> Callable:
     return date_filter
 
 
-def path_walk(top, topdown = False, followlinks = False):
+def path_walk(top, prefilter='.*', topdown = False, followlinks = False):
     """pendant for os.walk
     """
-    
-    names = list(top.iterdir())
 
+
+    current_level = len(str(top).split('/'))
+
+    filter_at_level = '/'.join(prefilter.split('/')[:current_level])
+    regex = re.compile(filter_at_level)
+    #print(current_level, filter_at_level)
+    #print(str(top).split('/'))
+    #print(str(prefilter).split('/'))
+    #stime = time.time()
+    #names = list(top.iterdir())
+    #print(" {:5.3f}s".format(time.time() - stime))
+
+    #stime = time.time()
+    names = [f for f in top.iterdir() if regex.search(str(f))]
+    #print(" {:5.3f}s".format(time.time() - stime))
+    #print(len(names), names[:30])
+    
     dirs = [node for node in names if node.is_dir() is True]
     nondirs = [node for node in names if node.is_dir() is False]
 
@@ -146,7 +168,7 @@ def path_walk(top, topdown = False, followlinks = False):
 
     for name in dirs:
         if followlinks or name.is_symlink() is False:
-            for x in path_walk(name, topdown, followlinks):
+            for x in path_walk(name, prefilter, topdown, followlinks):
                 yield x
 
     if topdown is not True:
@@ -247,7 +269,7 @@ class Connector_remote:
 
         #print("{:5.3f}s decode data".format(time.time() - starttime))
         starttime = time.time()
-        for k in ['ts', 'rg', 'vel', 'var', 'mask', 'vel_ch2', 'vel_ch3']:
+        for k in ['ts', 'rg', 'vel', 'var', 'mask', 'vel_ch2', 'vel_ch3', 'aux']:
             if k in data_container and type(data_container[k]) == list:
                 data_container[k] = np.array(data_container[k])
         logger.info("loaded data container from remote: {}".format(data_container.keys()))
@@ -270,6 +292,79 @@ class Connector_remote:
 
         return self.plain_dict
 
+
+def walk_str(pathinfo):
+    """match the names and subdirs with regex using string
+
+    only works for unix systems, but is reasonably fast 
+    
+    Args:
+        pathinfo: dict
+
+    Returns:
+        all_files
+    """
+
+    assert os.name == 'posix', 'walk_str only works with string based filepath'
+    
+    all_files = []
+    current_regex = pathinfo['matching_subdirs'] if 'matching_subdirs' in pathinfo else ''
+    current_re = re.compile(current_regex)
+    prefilter = pathinfo['base_dir'] + pathinfo['prefilter_subdirs'] if 'prefilter_subdirs' in pathinfo else '.*'
+
+    for root, d, files in os.walk(pathinfo['base_dir'], topdown=True):
+        #print('walk ', root, len(list(files)), files[:10])
+        root = root[:-1] if (root[-1] == '/') else root
+        current_level = len(root.split('/'))
+        filter_at_level = '/'.join(prefilter.split('/')[:current_level+1])
+        regex = re.compile(filter_at_level)
+        #print('root           ', root)
+        #print('filter at level', filter_at_level)
+        #print(str(root).split('/'))
+        #print(str(prefilter).split('/'))
+        #print('d before', len(d), d)
+        #print([f"{root}/{f}" for f in d])
+        d[:] = [f for f in d if regex.search(f"{root}/{f}")]
+        #print('d after', len(d), d)
+        #abs_filepaths = [f for f in files if re.search(current_regex, str(f))]
+        #abs_filepaths = [f for f in files if current_re.search(f)]
+        abs_filepaths = [f"{root}/{f}" for f in files if current_re.search(f"{root}/{f}")]
+        #logger.debug("valid_files {} {}".format(root, [f for f in files if re.search(current_regex, str(f))]))
+        #print("skipped_files {} {}".format(root, [f for f in files if not re.search(current_regex, str(f))]))
+        all_files += abs_filepaths
+        #files = [f for f in os.listdir('.') if re.match(r'[0-9]+.*\.jpg', f)]
+    return all_files
+
+
+
+def walk_pathlib(pathinfo):
+    """match the names and subdirs with regex using pathlib
+    
+    should give cross-platform compatability, but comes with a performance penalty (>3x)
+    
+    Args:
+        pathinfo: dict
+
+    Returns:
+        all_files
+    """
+    
+    all_files = []
+    current_regex = pathinfo['matching_subdirs'] if 'matching_subdirs' in pathinfo else ''
+    current_re = re.compile(current_regex)
+    prefilter = pathinfo['base_dir'] + pathinfo['prefilter_subdirs'] if 'prefilter_subdirs' in pathinfo else '.*'
+    for root, _, files in path_walk(Path(pathinfo['base_dir']), prefilter):
+        print('walk ', root, len(list(files)), files[:10])
+        #abs_filepaths = [f for f in files if re.search(current_regex, str(f))]
+        abs_filepaths = [f for f in files if current_re.search(str(f))]
+        #logger.debug("valid_files {} {}".format(root, [f for f in files if re.search(current_regex, str(f))]))
+        #print("skipped_files {} {}".format(root, [f for f in files if not re.search(current_regex, str(f))]))
+        all_files += abs_filepaths
+        #files = [f for f in os.listdir('.') if re.match(r'[0-9]+.*\.jpg', f)]
+    return all_files
+
+
+
 class Connector:
     """connect the data (from the ncfiles/local sources) to larda
 
@@ -301,17 +396,10 @@ class Connector:
 
         filehandler = {}
         for key, pathinfo in pathdict.items():
-            all_files = []
-            current_regex = pathinfo['matching_subdirs'] if 'matching_subdirs' in pathinfo else ''
 
             # 1. match the names and subdirs with regex
-            for root, _, files in path_walk(Path(pathinfo['base_dir'])):
-                #print('walk ', root, dirs, len(list(files)))
-                abs_filepaths = [f for f in files if re.search(current_regex, str(f))]
-                logger.debug("valid_files {} {}".format(root, [f for f in files if re.search(current_regex, str(f))]))
-                #print("skipped_files {} {}".format(root, [f for f in files if not re.search(current_regex, str(f))]))
-                all_files += abs_filepaths
-                #files = [f for f in os.listdir('.') if re.match(r'[0-9]+.*\.jpg', f)]
+            #all_files = walk_pathlib(pathinfo)
+            all_files = walk_str(pathinfo)
     
             # remove basedir (not sure if that is a good idea)
             all_files = [str(p).replace(pathinfo['base_dir'], "./") for p in all_files]
@@ -353,8 +441,11 @@ class Connector:
     def load_filehandler(self, path, camp_name):
         """load the filehandler from the json file"""
         filename = "connector_{}.json".format(self.system)
+        starttime = time.time()
         with open(path+'/'+camp_name+'/'+filename) as json_data:
                 self.filehandler = json.load(json_data)
+        logger.info("read in json filehandler {}: {}".format(self.system, time.time() - starttime))
+
 
     def collect(self, param, time_interval, *further_intervals, **kwargs) -> dict:
         """collect the data from a parameter for the given intervals
@@ -395,7 +486,7 @@ class Connector:
         #[print(e, (e[0][0] <= begin and e[0][1] > begin), (e[0][0] > begin and e[0][1] < end), (e[0][0] <= end and e[0][1] >= end)) for e in flist]
 
         load_data = setupreader(paraminfo)
-        datalist = [load_data(base_dir + e[1], time_interval, *further_intervals) for e in flist]
+        datalist = [load_data(Path(base_dir + e[1]), time_interval, *further_intervals) for e in flist]
         # [print(e.keys) if e != None else print("NONE!") for e in datalist]
         # reader returns none, if it detects no data prior to begin
         # now these none values are filtered from the list
@@ -407,6 +498,28 @@ class Connector:
         return data
 
 
+
+    def collect_path(self, param, time_interval, *further_intervals, **kwargs) -> dict:
+        """"
+        
+        Returns:
+            data_container
+        """
+        assert 'paths' in kwargs, "Without filepaths reading is tricky"
+        flist = kwargs['paths']
+        flist = [Path(f) if  type(f) == str else f for f in flist]
+
+        paraminfo = self.system_info["params"][param]
+        load_data = setupreader(paraminfo)
+        datalist = [load_data(e, time_interval, *further_intervals) for e in flist]
+        assert len(datalist) > 0, 'No data found for parameter: {}'.format(param)
+        datalist = list(filter(lambda x: x != None, datalist))
+        #Transf.join(datalist[0], datalist[1])
+        data = functools.reduce(Transf.join, datalist)
+
+        return data
+
+
     def description(self, param) -> str:
         paraminfo = self.system_info["params"][param]
         #print('connector local paraminfo: ' + paraminfo['variable_name'])
